/*
 * Copyright 2017 aquenos GmbH.
 * All rights reserved.
 * 
 * Please refer to the file NOTICE.txt in the root of the source distribution or
 * in the META-INF directory of the binary archive for licensing information.
 */

package com.aquenos.epics.jackie.common.exception;

import com.aquenos.epics.jackie.common.util.ListenerLockPolicy;

/**
 * <p>
 * Indicates that a listener has been removed while a notification was in
 * progress. This exception is typically thrown by a method that removes a
 * listener registration when the {@link ListenerLockPolicy#REPORT REPORT}
 * listener lock-policy is in use.
 * </p>
 * 
 * <p>
 * When this policy is used, this exception indicates that another thread was
 * just in the progress of notifying the removed listener and the notification
 * of the listener is thus going to happen even though the listener has been
 * removed. However, the fact that the exception has been thrown gives no
 * information about when this notification is going to happen. It could happen
 * after the exception is thrown or could already have happened before the
 * method that threw the exception returned.
 * </p>
 * 
 * @since EPICS Jackie 2.0.0
 * @author Sebastian Marsching
 */
public class ConcurrentNotificationException extends RuntimeException {

    private static final long serialVersionUID = 6129232965795754274L;

    /**
     * Constructs a new exception with <code>null</code> as its detail message.
     * The cause is not initialized, and may subsequently be initialized by a
     * call to {@link #initCause}.
     */
    public ConcurrentNotificationException() {
        super();
    }

    /**
     * Constructs a new exception with the specified detail message. The cause
     * is not initialized, and may subsequently be initialized by a call to
     * {@link #initCause}.
     * 
     * @param message
     *            the detail message. The detail message is saved for later
     *            retrieval by the {@link #getMessage()} method.
     */
    public ConcurrentNotificationException(String message) {
        super(message);
    }

    /**
     * Constructs a new exception with the specified cause and a detail message
     * of <code>(cause==null ? null : cause.toString())</code> (which typically
     * contains the class and detail message of <code>cause</code>). This
     * constructor is useful for exceptions that are little more than wrappers
     * for other throwables.
     * 
     * @param cause
     *            the cause (which is saved for later retrieval by the
     *            {@link #getCause()} method). (A <code>null</code> value is
     *            permitted, and indicates that the cause is nonexistent or
     *            unknown.)
     */
    public ConcurrentNotificationException(Throwable cause) {
        super(cause);
    }

    /**
     * Constructs a new exception with the specified detail message and cause.
     * Note that the detail message associated with <code>cause</code> is not
     * automatically incorporated in this exception's detail message.
     * 
     * @param message
     *            the detail message (which is saved for later retrieval by the
     *            {@link #getMessage()} method).
     * @param cause
     *            the cause (which is saved for later retrieval by the
     *            {@link #getCause()} method). (A <code>null</code> value is
     *            permitted, and indicates that the cause is nonexistent or
     *            unknown.)
     */
    public ConcurrentNotificationException(String message, Throwable cause) {
        super(message, cause);
    }

}
